package main

import (
	"bot/api"
	"bufio"
	"encoding/gob"
	"fmt"
	"log"
	"os"
	"runtime/debug"
	"strconv"
	"strings"

	"github.com/boltdb/bolt"
	tgbotapi "github.com/go-telegram-bot-api/telegram-bot-api/v5"
	"gopkg.in/yaml.v2"
)

var filename = "bot.map"

type Config struct {
	Account struct {
		Mode     string `yaml:"mode"` // pooling or webhook
		Token    string `yaml:"token"`
		Owner    int64  `yaml:"owner"`
		Endpoint string `yaml:"endpoint"`
		Port     int    `yaml:"port"`
	} `yaml:"account"`
}

var BotConfig Config
var bucketname = []byte("msg2chatid")
var db *bolt.DB
var lastreplyid int

func main() {
	logFile, err := os.OpenFile("bot.log", os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0644)
	if err != nil {
		fmt.Println("无法创建日志文件:", err)
		return
	}
	defer logFile.Close()
	log.SetOutput(logFile)
	yamlFile, err := os.ReadFile("bot.yaml")
	if err != nil {
		panic("read yaml fail: " + err.Error())
	}
	err = yaml.Unmarshal(yamlFile, &BotConfig)
	if err != nil {
		panic("unmarshal yaml fail: " + err.Error())
	}
	db, err = bolt.Open("bot.db", 0600, nil)
	if err != nil {
		panic("open db fail: " + err.Error())
	}
	defer db.Close()
	db.Update(func(tx *bolt.Tx) error {
		_, err := tx.CreateBucketIfNotExists([]byte(bucketname))
		if err != nil {
			panic("create bucket fail: " + err.Error())
		}
		return nil
	})
	go api.InitBot(BotConfig.Account.Mode, BotConfig.Account.Token, BotConfig.Account.Endpoint, BotConfig.Account.Port, handleUpdate)
	startCommandLine()
}

func deliverIncomingMsg(msg api.SimpleMsg) {
	log.Printf("receive message from %d %s\n", msg.ChatId, msg.Name)
	fmt.Printf("(%d)%s: %s\n:: ", msg.ChatId, msg.Name, msg.Text)
	lastreplyid = int(msg.ChatId)
	msgid := api.ForwardMsg(BotConfig.Account.Owner, msg.ChatId, msg.MessageID)
	db.Update(func(tx *bolt.Tx) error {
		b := tx.Bucket(bucketname)
		b.Put([]byte(strconv.Itoa(msgid)), []byte(strconv.Itoa(int(msg.ChatId))))
		log.Printf("store chatid %d for message %d\n", msg.ChatId, msgid)
		return nil
	})
	log.Printf("forward message from %d, message id %d\n", msg.ChatId, msgid)
}

func directmsg(msg api.SimpleMsg) {
	chatid := int(0)
	for i := 1; i < len(msg.Text); i++ {
		if msg.Text[i] == ' ' {
			chatid, _ = strconv.Atoi(msg.Text[1:i])
			msg.Text = msg.Text[i+1:]
			break
		}
	}
	if chatid == 0 {
		api.SendMsg(msg.ChatId, "format invaild")
		return
	}
	if msg.Text != "" {
		api.SendMsg(int64(chatid), msg.Text)
	}
}

func deliverOutgoingMsg(msg api.SimpleMsg) {
	if msg.Text != "" && msg.Text[0] == '*' {
		directmsg(msg)
		return
	}
	var storechatid int
	db.View(func(tx *bolt.Tx) error {
		b := tx.Bucket(bucketname)
		v := b.Get([]byte(strconv.Itoa(int(msg.ReplyID))))
		if v != nil {
			log.Printf("found chatid %s for message %d\n", string(v), msg.ReplyID)
			storechatid, _ = strconv.Atoi(string(v))
		} else {
			storechatid = 0
		}
		return nil
	})
	if storechatid == 0 || storechatid == int(msg.ChatId) {
		api.SendMsg(msg.ChatId, "reply to forward ...")
	} else {
		if msg.Text != "" {
			fmt.Printf("(%d)%s\n", storechatid, msg.Text)
			api.SendMsg(int64(storechatid), msg.Text)
		} else if msg.PhotoID != "" {
			api.SendExistingPhoto(int64(storechatid), msg.PhotoID)
		} else if msg.VideoID != "" {
			api.SendExistingVideo(int64(storechatid), msg.VideoID)
		}
	}
}

func deliverOutgoingMsgCmdLine(replyid int, text string) {
	fmt.Printf("(%d)%s\n", replyid, text)
	api.SendMsg(int64(replyid), text)
}

func commander(msg api.SimpleMsg) {
}

func handleUpdate(update tgbotapi.Update) {
	defer func() {
		if r := recover(); r != nil {
			log.Printf("Recovered from panic in handleUpdate: %v\n", r)
			api.SendMsg(BotConfig.Account.Owner, "Panic in handleUpdate! Check the log for details.")
			debug.PrintStack()
		}
	}()

	msg := api.FormatMsg(update)
	if msg.Type != "private" {
		return
	}
	if msg.Text != "" {
		if msg.Text[0] == '/' {
			if msg.FromID == BotConfig.Account.Owner {
				commander(msg)
			}
			return
		}
	}

	if msg.FromID != BotConfig.Account.Owner {
		deliverIncomingMsg(msg)
	} else {
		deliverOutgoingMsg(msg)
	}
}

func SaveMapToDisk(m map[int]int64) error {
	file, err := os.Create(filename)
	if err != nil {
		return err
	}
	defer file.Close()

	encoder := gob.NewEncoder(file)
	err = encoder.Encode(m)
	if err != nil {
		return err
	}

	return nil
}

func LoadMapFromDisk() (map[int]int64, error) {
	file, err := os.Open(filename)
	if err != nil {
		return make(map[int]int64), err
	}
	defer file.Close()

	decoder := gob.NewDecoder(file)
	var m map[int]int64
	err = decoder.Decode(&m)
	if err != nil {
		return make(map[int]int64), err
	}
	return m, nil
}

func startCommandLine() {
	reader := bufio.NewReader(os.Stdin)
	for {
		fmt.Print(":: ")
		text, _ := reader.ReadString('\n')
		doCommand(text)
	}
}

func parseCommand(text string) (string, []string) {
	cmdarr := strings.Split(text, " ")
	cmd := cmdarr[0]
	args := cmdarr[1:]
	return cmd, args
}

func isNumber(s string) bool {
	_, err := strconv.Atoi(s)
	return err == nil
}

func doCommand(text string) {
	if text == "" {
		return
	}
	cmd, args := parseCommand(text)
	if cmd == "!" || cmd=="0" {
		deliverOutgoingMsgCmdLine(lastreplyid, args[0])
	} else if isNumber(cmd) {
		chatid, _ := strconv.Atoi(cmd)
		api.SendMsg(int64(chatid), args[0])
s	} else {
		fmt.Println("unknown command")
	}
}

